package de.ozgcloud.eingang.fim;

import de.ozgcloud.eingang.common.formdata.FormData;
import de.ozgcloud.eingang.common.formdata.FormHeader;
import de.ozgcloud.eingang.common.formdata.IncomingFile;
import de.ozgcloud.eingang.semantik.enginebased.EngineBasedSemantikAdapter;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.w3c.dom.Document;
import org.xml.sax.SAXException;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import java.io.IOException;
import java.util.Optional;

@Log4j2
@Component
public class FimBasedAdapter implements EngineBasedSemantikAdapter {

    static final String FIM_FORM_ENGINE_NAME = "FIM";

    @Autowired
    private FimService fimService;

    @Override
    public boolean isResponsible(final FormData formData) {
        final String formEngineName = formData.getHeader().getFormEngineName();
        return FIM_FORM_ENGINE_NAME.equals(formEngineName);
    }

    @Override
    public FormData parseFormData(final FormData initialFormData) {
        LOG.info("FIM Mapper running to map formData");
        final Optional<String> entryPointOpt = getEntryPoint(initialFormData.getHeader());
        if (entryPointOpt.isEmpty()) {
            LOG.error("No entry point found in metadata file for fim data mapping");
            return initialFormData;
        }
        final Optional<IncomingFile> fileOpt = findFile(initialFormData, entryPointOpt.get());
        if (fileOpt.isEmpty()) {
            LOG.error("Entry point file not found for fim data mapping");
            return initialFormData;
        }
        try {
            final Document document = loadDocument(fileOpt.get());
            return fimService.transformDocument(document, initialFormData);
        } catch (ParserConfigurationException | SAXException | IOException | FimException e) {
            LOG.error("Can't transform document into fim formdata");
            return initialFormData;
        }
    }

    private Optional<IncomingFile> findFile(final FormData formData, final String name) {
        for (IncomingFile file : formData.getRepresentations()) {
            if (file.getName().endsWith(name)) {
                return Optional.of(file);
            }
        }
        return Optional.empty();
    }

    private Optional<String> getEntryPoint(final FormHeader formHeader) {
        // FIXME OZG-5778 sollte über formdata metadaten kommen statt aus dem meta file geparsed zu werden
        return Optional.of("Antrag.xml");
    }

    private Document loadDocument(final IncomingFile incomingFile) throws ParserConfigurationException, IOException, SAXException {
        final DocumentBuilder builder = DocumentBuilderFactory.newInstance().newDocumentBuilder();
        return builder.parse(incomingFile.getContentStream());
    }
}
