package de.ozgcloud.eingang.fim;

import de.ozgcloud.eingang.common.formdata.FormData;
import de.ozgcloud.eingang.common.formdata.IncomingFile;
import de.ozgcloud.eingang.common.formdata.IncomingFileGroup;
import lombok.extern.log4j.Log4j2;
import org.apache.commons.lang3.function.TriFunction;
import org.springframework.stereotype.Service;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Text;

import java.util.*;

@Service
@Log4j2
class FimDataMapper implements TriFunction<Document, FimScheme, FormData, FormData.FormDataBuilder> {

    private static final String LABEL_KEY = "label";
    private static final String VALUE_KEY = "value";

    @Override
    public FormData.FormDataBuilder apply(final Document document, final FimScheme fimScheme, final FormData initialFormData) {
        final FormData.FormDataBuilder formDataBuilder = FormData.builder();
        Map<String, Object> data = new LinkedHashMap<>();
        process(document.getDocumentElement(), fimScheme, data, 0);
        formDataBuilder.formData(data);
        formDataBuilder.attachments(initialFormData.getAttachments());
        formDataBuilder.representations(initialFormData.getRepresentations());
        return formDataBuilder;
    }

    private void process(final Element element, final FimScheme fimScheme, final Map<String, Object> data, final int level) {
        final NodeList childNodes = element.getChildNodes();

        LOG.debug(">".repeat(level) + " " + element.getNodeName());

        for(int i = 0; i < childNodes.getLength();i++) {
            final Node child = childNodes.item(i);
            if (!(child instanceof Element)) {
                continue;
            }
            if (child.getChildNodes().getLength() == 1 && child.getChildNodes().item(0) instanceof Text textNode) {
                insertValueIntoFormData(data, fimScheme.getFieldName(child.getNodeName()), child.getNodeName(), textNode.getTextContent());
            } else {
                final Map<String, Object> childMap = new LinkedHashMap<>();
                insertValueIntoFormData(data, fimScheme.getFieldName(child.getNodeName()), child.getNodeName(), childMap);
                process((Element) child, fimScheme, childMap, level + 1);
            }
        }
    }

    private void insertValueIntoFormData(final Map<String, Object> data, final Optional<String> fieldName, final String nodeName, final Object obj) {
        final Map<String, Object> labelMap = Map.of(LABEL_KEY, fieldName.orElse(nodeName), VALUE_KEY, obj);
        data.put(nodeName, labelMap);
    }
}
