/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.eingang.semantik.enginebased.formsolutions;

import org.springframework.stereotype.Component;

import de.ozgcloud.eingang.common.formdata.Antragsteller;
import de.ozgcloud.eingang.common.formdata.FormData;
import de.ozgcloud.eingang.common.formdata.FormDataUtils;

@Component
class FormSolutionsAntragstellerMapper implements FormSolutionsEngineBasedMapper {

	public static final String ANTRAGSTELLER_PANEL_IDENTIFIER = "Antragstellende Person";

	public static final String POSTKORBHANDLE = "postkorbhandle";

	public static final String VORNAME_KEY = "AS_Vorname";
	public static final String NACHNAME_KEY = "AS_Name";

	@Override
	public FormData parseFormData(FormData formData) {
		return FormDataUtils.from(formData)
				.builder()
				.antragsteller(buildAntragsteller(formData))
				.build();
	}

	protected Antragsteller buildAntragsteller(FormData formData) {
		var antragstellerData = IdentifierValueParser.parsePanelsData(formData);

		return Antragsteller.builder()
				.postfachId(getPostkorbhandle(formData))
				.vorname(antragstellerData.get(VORNAME_KEY))
				.nachname(antragstellerData.get(NACHNAME_KEY))
				.build();
	}

	private String getPostkorbhandle(FormData formData) {
		return (String) formData.getFormData().get(POSTKORBHANDLE);
	}
}