package de.ozgcloud.eingang.semantik.enginebased.dfoerdermittel;

import static java.util.stream.Collectors.*;

import java.util.Map;
import java.util.Objects;
import java.util.function.Predicate;
import java.util.stream.IntStream;

import de.ozgcloud.eingang.common.formdata.FormData;
import de.ozgcloud.eingang.common.formdata.FormHeaderTestFactory;
import lombok.Builder;

class DFoerdermittelFormDataTestFactory {

	static final String FORM_NAME = "Name of form";
	static final String FORM_ID = "Wert von ProjectTitle";
	static final String POSTFACH_ID = "4dd01647-b9d9-4775-1b50-08da3d83800a";
	static final String ORGANISATIONS_EINHEIT_ID = "9795669";

	static Fachnachricht.FachnachrichtBuilder createFachnachrichtBuilder() {
		return Fachnachricht.builder()
				.title(FORM_NAME)
				.projectTitle(FORM_ID)
				.postfachId(POSTFACH_ID)
				.organisationsEinheitId(ORGANISATIONS_EINHEIT_ID);
	}

	static FormData create() {
		return createBuilder().build();
	}

	static FormData.FormDataBuilder createBuilder() {
		return createBuilderWithFachnachricht(createFachnachrichtBuilder().build());
	}

	static FormData.FormDataBuilder createBuilderWithFachnachricht(Fachnachricht fachnachricht) {
		return FormData.builder()
				.header(FormHeaderTestFactory.create())
				.formData(Map.of("Fachnachricht", createFachnachrichtMapWithFachnachricht(fachnachricht)));
	}

	static Map<String, Object> createFachnachrichtMap() {
		return createFachnachrichtMapWithFachnachricht(createFachnachrichtBuilder().build());
	}

	static Map<String, Object> createFachnachrichtMapWithFachnachricht(Fachnachricht fachnachricht) {
		return createMapWithValueFilter(
				Objects::nonNull,
				"Title", fachnachricht.title(),
				"ProjectTitle", fachnachricht.projectTitle(),
				"InboxReference", "sh/sh/" + fachnachricht.postfachId(),
				"MetaText1", fachnachricht.organisationsEinheitId()
		);
	}

	private static Map<String, Object> createMapWithValueFilter(Predicate<String> valuePredicate, String... keyValuePairs) {
		if (keyValuePairs.length % 2 != 0) {
			throw new IllegalArgumentException("Array length must be even to form key-value pairs.");
		}

		return IntStream.range(0, keyValuePairs.length / 2)
				.boxed()
				.filter(i -> valuePredicate.test(keyValuePairs[2 * i + 1]))
				.collect(toMap(i -> keyValuePairs[2 * i], i -> keyValuePairs[2 * i + 1]));
	}

	@Builder
	record Fachnachricht(String title, String projectTitle, String postfachId, String organisationsEinheitId) {
		FormData createFormData() {
			return createBuilderWithFachnachricht(this).build();
		}
	}
}
