package de.ozgcloud.eingang.xta.zip;

import java.io.FilterInputStream;
import java.io.IOException;
import java.io.InputStream;

public class LimitedInputStream extends FilterInputStream {
	static final String LIMITED_EXCEEDED_MESSAGE = "Read limit exceeded";

	private final long maxSize;
	long bytesRead;

	public LimitedInputStream(InputStream in, long maxSize) {
		super(in);
		this.maxSize = maxSize;
		this.bytesRead = 0;
	}

	@Override
	public int read() throws IOException {
		var byteValue = super.read();
		if (byteValue != -1) {
			updateAndVerifyReadLimit(1);
		}
		return byteValue;
	}

	@Override
	public int read(byte[] b, int off, int len) throws IOException {
		return updateAndVerifyReadLimit(super.read(b, off, len));
	}

	private int updateAndVerifyReadLimit(int newBytesRead) throws IOException {
		if (newBytesRead != -1) {
			bytesRead += newBytesRead;
			if (bytesRead > maxSize) {
				throw new IOException(LIMITED_EXCEEDED_MESSAGE);
			}
		}
		return newBytesRead;
	}
}
